#include <qwidget.h>
#include <qpainter.h>
#include <qpixmap.h>

#include <assert.h>

#include "sampleview.h"
#include "sample.h"
#include "observer.h"

#include "error.h"

SampleView::SampleView(QWidget *parent, const char *name) 
  : QWidget(parent, name), Observer(NULL) { 

  sample = NULL;
  pixmap = new QPixmap(384, 192);
  pixmap->fill(QColor(0x00, 0xCC, 0xFF));

  buffers[0] = (float*)malloc(sizeof(float) * 384);
  buffers[1] = (float*)malloc(sizeof(float) * 384);

  bufferLen = 384;

  if (!buffers[0] || !buffers[1]) {
    error_message("out of mem!");
    exit(110);
  }

  setBackgroundMode(Qt::NoBackground);

  selectionMode = 0;

  stepSize = 0;
}


void SampleView::update(Subject *s) {

  // only render current sample 
  if (s != sample) return;

  repaint();
}


void SampleView::renderPixmap() {

  QPainter *p = new QPainter(pixmap);
  pixmap->fill(QColor(0x00, 0xCC, 0xFF));

  if (!sample || !sample->isValid()) return;

  int numChannels = std::min(sample->getNumChannels(), SV_NUM_BUFFERS);
  int numFrames = sample->getNumFrames();

  sample->renderData(buffers, numChannels, 0, numFrames - 1, bufferLen);

  stepSize = ((float)bufferLen) / ((float)numFrames);

  p->setPen(Qt::green);

  int chHeight = height() / numChannels;
  int chOffset = chHeight >> 1;

  int y = chOffset;

  for (int j = 0; j < numChannels; j++) {
    for (int i = 0; i < bufferLen; i++) {
    
      p->drawLine(i, y, 
		  i, y + (int)(buffers[j][i] * 0.9f * chOffset));
    }

    y += chHeight;
  }
  
  p->end();
}

void SampleView::subjectDeleted(Subject *s) {

  if (s == sample) sample = NULL;
  update(NULL);
}


void SampleView::setSample(Sample *s) {

  if (sample) sample->removeObserver(this);
  sample = s;
  if (sample) sample->addObserver(this);

  renderPixmap();
  repaint();
}

void SampleView::paintEvent(QPaintEvent *e) {

  QPainter *p = new QPainter(this);

  p->drawPixmap(0, 0, *pixmap); 

  if (!sample || !sample->isValid()) { p->end(); return; }

  p->setPen(Qt::red);

  p->drawLine((int)(sample->getSampleStart() * stepSize), 0,
	      (int)(sample->getSampleStart() * stepSize), height() - 1);
  p->drawLine((int)(sample->getSampleEnd() * stepSize), 0,
	      (int)(sample->getSampleEnd() * stepSize), height() - 1);
  
  p->setPen(Qt::blue);

  p->drawLine((int)(sample->getLoopStart() * stepSize), 0,
	      (int)(sample->getLoopStart() * stepSize), height() - 1);
  p->drawLine((int)(sample->getLoopEnd() * stepSize), 0,
	      (int)(sample->getLoopEnd() * stepSize), height() - 1);

  p->end();
}


void SampleView::mousePressEvent(QMouseEvent *e) {

  if (stepSize == 0) stepSize = 1;

  int p  = (int)((float)e->x() / stepSize);

  if ((p >= 0) && (p < sample->getNumFrames())) {
    
    switch(selectionMode) {
      
    case 0: // loop
      
      switch (e->button()) {

      case LeftButton:
	sample->setLoopStart(p);
	break;

      case RightButton:
	sample->setLoopEnd(p);
	break;
      }

      break;

    case 1: // trim
      
      switch (e->button()) {

      case LeftButton:
	sample->setSampleStart(p);
	break;
	
      case RightButton:
	sample->setSampleEnd(p);
	break;
      }
      
      break;
    }
  }
}  


void SampleView::setSelectionMode(int v) {

  selectionMode = v;
}

void SampleView::setLoop() {

  selectionMode = 0;
}

void SampleView::setTrim() {

  selectionMode = 1;
}

